from com.ibm.websphere.management import AdminClientFactory, AdminClient
from pytwss.mbean.mbean_utils import getAdminClient

#
# Gets information about the deployment manager, such as the cell, node
# process, server, and domain names.
# return: the DeploymentMngrInfo
#
def getDeploymentMngrInfo():
	#Use the Admin Client to obtain the DM Server MBean
	admin_client = getAdminClient()
	server = admin_client.getServerMBean()

	#Get all the DM Properties(cell, node, process, etc)
	cellName = server.getKeyProperty("cell")
	nodeName = server.getKeyProperty("node")
	processName = server.getKeyProperty("process")
	serverName = server.getKeyProperty("name");
	
	#Get the domain name (WebSphere
	domainName = admin_client.getDomainName()
	
	return __DeploymentMngrInfo(cellName, nodeName, processName, serverName, domainName)

#
# __DeploymentMngrInfo can be queried to retrieve the dm server information, 
# including the cell name, node name, process name, server name, and domain
# name of the deployment manager server.
#
class __DeploymentMngrInfo:
	#
	# Constructs a __DeploymentMngrInfo object.
	# param: cellName the cell name of the dm
	# param: nodeName the node name of the dm
	# param: processName the process name of the dm
	# param: serverName the server name of the dm
	# param: domainName the domain name of the dm
	def __init__(self, cellName, nodeName, processName, serverName, domainName):
   		self.__cellName = cellName
		self.__nodeName = nodeName
		self.__processName = processName
		self.__serverName = serverName
		self.__domainName = domainName

	def getCellName(self):
		return self.__cellName

	def getNodeName(self):
		return self.__nodeName
	
	def getProcessName(self):
		return self.__processName
		
	def getServerName(self):
		return self.__serverName
	
	def getDomainName(self):
		return self.__domainName
		
	def toString(self):
		deploy_info_str = []
		deploy_info_str.append("Cell Name: %s" % self.__cellName)
		deploy_info_str.append("Node Name: %s" % self.__nodeName)
		deploy_info_str.append("Server Name: %s" % self.__serverName)
		deploy_info_str.append("Domain Name: %s" % self.__domainName)
		return ", ".join(deploy_info_str)
		
