from com.ibm.websphere.management import AdminClientFactory, AdminClient
from pytwss.mbean.mbean_utils import getAdminClient, MBeanProxy, MAX_RETRIES, Scope
from pytwss.utils import log
import com.ibm.ws.management.filetransfer.client.FileTransferOptionsImpl as FileTransferOptions


class FileTransferAdministration(MBeanProxy):
	def __init__(self, nodeName, max_retries=MAX_RETRIES):
		scope = Scope(type='FileTransferClient', node=nodeName)
		MBeanProxy.__init__(self, scope,
		                    max_retries=max_retries)
		      
#
# Transfers a file located in the DM's config/temp directory
# to a node.
# param: sourcePath - The path of the source file relative to Dmgr01/config/temp
# param destFile - The full path of the destination java.io.File
# param nodeName - specifies the node name to transfer the file to
# param isSecure - specifies whether this is a secure transfer
# param compress - specifies whether or not to compress the file
# param overwrite - specifies whether or not to overwrite the file
#        
def downloadFile(sourcePath, destFile, nodeName, isSecure, compress=0, overwrite=1):
	log("Transfering %s to node %s:%s, isSecure=%s, compress=%s, overwrite=%s" % (sourcePath, nodeName, destFile, isSecure, compress, overwrite))
	fileTransferBean = FileTransferAdministration(nodeName)
	fto = FileTransferOptions()
	fto.setDeleteSourceOnCompletion(0)
	fto.setSecure(isSecure)
	fto.setCompress(compress)
	fto.setOverwrite(overwrite)
	fileBean = fileTransferBean.ref
	parms = [sourcePath, destFile, fto]
	sigs = ['java.lang.String', 'java.io.File', 'com.ibm.websphere.management.filetransfer.client.FileTransferOptions']
        log("fileBean %s" %(fileBean))          # need for debugging an intermittent mysterious problem where name= is not defined
	getAdminClient().invoke(fileBean, "downloadFile", parms, sigs)
	              
def removeFile(sourceFile, destPath, nodeName, isSecure):
	log("Removing %s from node %s to %s, isSecure=%s" % (sourceFile, nodeName, destPath, isSecure, compress, overwrite))
	fileTransferBean = FileTransferAdministration(nodeName)
	fto = FileTransferOptions()
	fto.setDeleteSourceOnCompletion(1)
	fto.setSecure(isSecure)
	fto.setCompress(0)
	fto.setOverwrite(1)
	fileBean = fileTransferBean.ref
	parms = [sourceFile, destPath, fto]
	sigs = ['java.io.File', 'java.lang.String', 'com.ibm.websphere.management.filetransfer.client.FileTransferOptions']
	getAdminClient().invoke(fileBean, "uploadFile", parms, sigs)