#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2000, 2006
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

from com.ibm.websphere.management import AdminClientFactory, AdminClient
from javax.management import ReflectionException, MBeanException, \
                             AttributeNotFoundException, InstanceNotFoundException
from javax.management import Attribute
from com.ibm.websphere.management.exception import ConnectorException
from java.util import Properties

from pytwss.utils import _, log

import com.ibm.ws.scripting.AdminControlClient
AdminControl = com.ibm.ws.scripting.AdminControlClient.getInstance()

# Access admin client APIs directly for more accurate JMX reporting
__admin_client = None
MAX_RETRIES = 5 # Default retries before giving up on transient errors

__admin_client_username = None
__admin_client_password = None

def getAdminClient():
   """Gets a reference to the AdminClient."""
   global __admin_client
   global __admin_client_username
   global __admin_client_password
   if not __admin_client:
      conn_props = Properties()
      conn_props.setProperty(AdminClient.CONNECTOR_TYPE, AdminControl.getType())
      conn_props.setProperty(AdminClient.CONNECTOR_HOST, AdminControl.getHost())
      conn_props.setProperty(AdminClient.CONNECTOR_PORT, AdminControl.getPort())
      log("getAdminClient user=%s" %(__admin_client_username))
      if __admin_client_username != None:
         conn_props.setProperty(AdminClient.USERNAME, __admin_client_username)
      if __admin_client_password != None:
         conn_props.setProperty(AdminClient.PASSWORD, __admin_client_password)
      __admin_client = AdminClientFactory.createAdminClient(conn_props)
      log("AdminClient %s" %(__admin_client))
   return __admin_client
        
def setUserName(userName):
   global __admin_client_username
   __admin_client_username = userName
   
def setPassword(password):
   global __admin_client_password
   __admin_client_password = password

def locateMBean(scope):
	"""Locates an MBean object name for the specified scope.

	The scope object may either be a 'Scope' object, or a query
	string. The scope object will be converted to a query string
	representation prior to locating the object. Once an MBean
	is found, its attributes are queried to ensure the MBean is
	live."""
	mbean_name = None
	queryString = str(scope)
        for j in range(3):
           if mbean_name == None:
      	       mbean_list = AdminControl.queryMBeans(queryString)
      	       for i in range(0, mbean_list.size()):
      		  inst = mbean_list[i]
      		  obj_name = inst.getObjectName()
      		  try:
      			attribs = AdminControl.getAttributes(
      			    str(obj_name), None)
      		  except:
      			continue
      		  mbean_name = obj_name
      		  break
	log(_("SOAC7500I") %{ '1' : queryString, '2' : mbean_name })
	return mbean_name

class Scope:
	"""WebSphere JMX scope object.

	JMX allows scoping of objects through a list of attribute/values
	that indicate how to locate the object. WebSphere has a set of
	conventional attribute names that describe elements of the
	WAS topology. Attribute names also contain MBean-related
	identifiers. This class allows for an instance to be constructed
	using bean-like key-value pair assign. The instance can then
	be treated like a dictionary. When converted to a string
	representation, an equivalent JMX query string is produced."""

	def __init__(self, *args, **kwargs):
		"""Initializes the scope with the supplied scope attributes."""
		self.scope = kwargs
		self.wildcard = 1

	def __getitem__(self, key):
		"""Gets a scope property when acting as a dict."""
		return self.scope[key]

	def __setitem__(self, key, value):
		"""Sets a scope property when acting as a dict."""
		self.scope[key] = value

	def __delitem__(self, key):
		"""Removes a scope property when acting as a dict."""
		del self.scope[key]

	def has_key(self, key):
		"""Returns a boolean indicating whether this key exists in
		the scope properties."""
		return self.scope.has_key(key)

	def keys(self):
		"""Returns a list of scope attributes."""
		return self.scope.keys()

	def setUseWildcard(self, val):
		"""Indicates whether to use a wildcard representation of
		a query string when the scope is converted to string.

		Setting this to false assumes that all property attributes
		are provided in the scope."""
		self.wildcard = val

	def getUseWildcard(self):
		"""Indicates whether wildcards are being used in constructing
		the query string."""
		return self.wildcard

	def __str__(self):
		"""Converts the scope property set into a JMX query.

		Since WebSphere objects typically have many different objects
		beyond, wildcards are appended (default behavior - can be
		disabled by setUseWildcard(0)) to the query string."""
		result = '*'
		buffer = [ ]
		for key in self.scope.keys():
			buffer.append("%s=%s" %(key, self.scope[key]))
		if len(buffer) > 0:
			result = ','.join(buffer)
			if self.getUseWildcard():
				result += ',*'
		return result

class RecoverableJMXProxy:
	"""Recoverable JMX execution proxy for manipulating remove JMX
	objects.

	This proxy maintains a reference to a live MBean object. If an error
	occurs communicating with that object, then the proxy will automatically
	attempt to locate another live (and equivalent per the scope) object
	to execute the operation on. This is done transparently provided the
	'ex()' function is called -which works similar to apply."""

	def __init__(self, scope, initial_ref=None, max_retries=MAX_RETRIES):
		"""Initializes the recoverableJMXProxy instance.

		The scope object references either a 'Scope' class object, or a
		scope query string. If an initial reference is provided, it will
		be used as the first reference before searching for additional
		MBean references. The max retries indicates how many times to
		retry locating a valid MBean before giving up."""
		if scope is None:
			raise ValueError, 'scope'
		self.scope = scope
		self.max_retries = MAX_RETRIES
		self.retry_cnt = 0
		self.admin_client = getAdminClient()
		if initial_ref is None:
			self.ref = locateMBean(self.scope)
		else:
			self.ref = initial_ref

	def ex(self, func, *args):
		"""Attempts to execute the supplied remove function object against
		the supplied arguments, retrying on failure."""
		while self.retry_cnt < self.max_retries:
			exception = None
			try:
				return apply(func, args)
			except ReflectionException, re:
				exception = re
			except InstanceNotFoundException, infe:
				exception = infe
			except:
				raise
			if exception is not None:
				log(_("SOAC7501W") %{ '1' : self.type,
				    '2' : exception.toString() })
				self.retry_cnt += 1
				self.ref = locateMBean(self.scope)
				if self.ref is None:
					break
		raise RuntimeError, _("SOAC7502E") %self.max_retries

class MBeanInvocationProxy(RecoverableJMXProxy):
	"""Proxy class for MBean operation invocations.

	This class proxies performing a dynamic Java invocation on a JMX
	defined operation. This class takes the operation definition and
	becomes callable as if it is that function. This class contains
	automatic retry functionality if communication with the MBean is
	severed."""

	def __init__(self, scope, function, sigs,
	             initial_ref=None, max_retries=MAX_RETRIES):
		"""Creates an operation proxy instance.

		The scope objet references either a 'Scope' class object, or a
		scope query string. The supplied function is the name of the
		Java method on the MBean to invoke. Sigs is an array of Java
		method signature types that correspond to the method parameters."""
		RecoverableJMXProxy.__init__(self, scope, initial_ref=initial_ref,
		                             max_retries=max_retries)
		self.func = function
		self.sigs = sigs

	def __call__(self, *args):
		"""Executes the proxied function with the supplied args."""
                #log("invoke %s %s %s %s %s" %(self.admin_client, self.ref, self.func, args, self.sigs))
                if self.func.startswith("__"):                          # cause exception, never want to invoke internal jython methods on the mbean
                   return None
		return self.ex(self.admin_client.invoke, self.ref, self.func,
		               args, self.sigs)

class MBeanProxy(RecoverableJMXProxy):
	"""Proxy class for working with a JMX MBean as through it were a native
	class object.

	The proxy class is initialized with a specified scope object to match.
	The first live object (in a cluster, this may be multiple MBean
	instances of thes same object) is chosen to execute JMX operations.
	This proxy provides two functions: 1) For MBean attributes, it exposes
	those attributes by behaving as a dictionary. The keys of the
	dictionary are the attributes, and setting/getting dictionary values
	interacts with the MBean attributes. 2) For MBean operations, those
	operations can be invoked on this class as though they were defined as
	native operations on this class. This class includes logic to perform
	the remote JMX notification, including retries upon failure of the
	referenced MBean instance."""

	def __init__(self, scope, max_retries=MAX_RETRIES):
		"""Creates an instance of the MBean proxy, picking the first
		MBean that matches the specified scope."""
		RecoverableJMXProxy.__init__(self, scope, max_retries=max_retries)
		self.inv_dict = { } # Invocation proxy cache

	def __getattr__(self, func):
		"""Called when performing an operation invocation for a remote
		MBean method.

		This function automatically creates an operation proxy if no
		proxy already exists, and locates the MBean if necessary. A
		proxy is created for each remote method."""
		if self.inv_dict.has_key(func):
			return self.inv_dict[func]
		sigs = self.__getParameterSigs(func)
		proxy = MBeanInvocationProxy(self.scope, func, sigs,
		    initial_ref=self.ref, max_retries=self.max_retries)
		self.inv_dict[func] = proxy
		return proxy

	def __getitem__(self, key):
		"""Returns the value of the specified MBean attribute when
		behaving as a dict.

		All values will be strings per the JMX API."""
		return self.ex(self.admin_client.getAttribute, self.ref, key)

	def __setitem__(self, key, value):
		"""Sets the value of teh specified MBean attribute when
		behaving as a dict.

		All values should be strings per the JMX API."""
		self.ex(self.admin_client.setAttribute, self.ref,
		        Attribute(key, value))

	def __getOperationInfo(self, op_name):
		"""Gets the MBean operation description for the specified
		operation name, using the current object reference."""
		info = None
		mbean_info = self.ex(self.admin_client.getMBeanInfo, self.ref)
		op_info_list = mbean_info.getOperations()
		for op_info in op_info_list:
			if op_info.getName() == op_name:
				info = op_info
				break
		return info

	def __getParameterSigs(self, op_name):
		"""Gets a list of method parameter types for the specified operation
		name, using the current object reference."""
		sigs = None
		op_info = self.__getOperationInfo(op_name)
		if op_info:
			sigs = [ x.getType() for x in op_info.getSignature() ]
		return sigs

	def has_key(self, key):
		"""Indicates whether this MBean has the specified attribute."""
		return key in self.keys()

	def keys(self):
		"""Returns a list of attribute names on the MBean instance."""
		mbean_info = self.ex(self.admin_client.getMBeanInfo, self.ref)
		attrib_list = mbean_info.getAttributes()
		attribs = [ x.getName() for x in attrib_list ] # Extract key
		return attribs
