#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2000, 2006
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

from pytwss.utils import _, log, boolToString

from javax.management import MBeanException

from com.ibm.twss.spm.admin.common import ServiceType
from com.ibm.twss.spm.admin.common import ServiceImplDefinition
from com.ibm.twss.spm.admin.common import ServiceDefinition
from com.ibm.twss.spm.admin.common import OperationDefinition
from com.ibm.twss.spm.admin.common import ScopedPolicies
from com.ibm.twss.spm.admin.common import ScopedPolicy
from com.ibm.twss.spm.admin.common import PolicyValue
from com.ibm.twss.spm.admin.common import PolicyType
import com.ibm.twss.spm.admin.svc.GetServiceImplementationsRequest \
	as SvcImplsReq
import com.ibm.twss.spm.admin.svc.CreateServiceImplementationRequest \
	as CrtImplReq
import com.ibm.twss.spm.admin.svc.CreateServiceRequest \
	as CrtSvcReq
import com.ibm.twss.spm.admin.svc.CreateOperationRequest \
	as CrtOpReq
import com.ibm.twss.spm.admin.policy.CreatePoliciesRequest \
	as CrtPolReq
import com.ibm.twss.spm.admin.type.CreatePolicyTypeRequest \
	as CrtTypeReq

from com.ibm.twss.spm.mbean import SPMJython


class ServicePolicyAccess:
	"""Service policy access interface."""

	def __init__(self):
		self.ref = 'ServicePolicyAccess'
                
        def getRef(self):
           return self.ref

class RequesterAdministration:
	"""Requester administration interface."""

	def __init__(self):
                self.ref = 'RequesterAdministration'
        def getRef(self):
           return self.ref

class ServiceAdministration:
	"""Service administration interface."""

	def __init__(self):
                self.ref = 'ServiceAdministration'
        def getRef(self):
           return self.ref

class SubscriptionAdministration:
	"""Subscription administration interface."""

	def __init__(self):
                self.ref = 'SubscriptionAdministration'
        def getRef(self):
           return self.ref

class PolicyAdministration:
	"""Policy administration interface."""

	def __init__(self):
		self.ref = 'PolicyAdministration'
        def getRef(self):
           return self.ref

class TypeAdministration:
	"""Type administration interface."""

	def __init__(self):
		self.ref = 'TypeAdministration'
        def getRef(self):
           return self.ref

def getMessageFromTargetEx(target):
	if target:
		if hasattr(target, 'getMessageID'):
			return target.getMessageID() + " " + target.getText()
		else:
			return target.toString()
	else:
		
                return _("SOAC7530E")


def registerServiceImplementation(svc_admin, svc_name, impl_name,
                                  enabled, desc):
	"""Registers a esrvice implementation.

	'svc_admin' is a reference to the service administration interface
	proxy instance. The remaining arguments are:
	    'svc_name'  : The name of the service
	    'impl_name' : The service implementation name
	    'enabled'   : Whether the service should be enabled
	    'desc'      : Description
	These arguments follow from the service administration interface."""
	log(_("SOAC7503I") %{ '1' : svc_name, '2' : impl_name,
	    '3' : boolToString(enabled), '4' : desc })
	should_add = 0
	impls_resp = None
        try:
           impls_resp = SPMJython.invoke(svc_admin.getRef(), "getServiceImplementations", 
		SvcImplsReq(service=svc_name))
     	except MBeanException, me:
     		svc_ex = me.getTargetException()
     		msgid = getMessageFromTargetEx(svc_ex)
     		log(_("SOAC7516E") %{ '1' : msgid })
        
	if impls_resp and impls_resp.definitions and \
	   impls_resp.definitions.definition is not None:
		existing = [ x.getImplName() for x in
		             impls_resp.definitions.definition ]
		should_add = impl_name not in existing
	else:
		should_add = 1 # No actual list yet

	if should_add:
		defn = ServiceImplDefinition(service=svc_name, implName=impl_name,
		                             enabled=enabled, description=desc)
		try:
			SPMJython.invoke(svc_admin.getRef(), "createServiceImplementation", CrtImplReq(definition=defn))
			added = 1
			log(_("SOAC7507I"))
		except MBeanException, me:
			svc_ex = me.getTargetException()
			msgid = getMessageFromTargetEx(svc_ex)
			log(_("SOAC7516E") %{ '1' : msgid })
	else:
		log(_("SOAC7508E"))

def policyToString(policy):
	"""Converts a policy data structure (as generated by WSDL2JAVA from the
	Policy Administration WSDL interface to a string."""
	value = None
	type = None
	if policy.value:
		value = policy.value.value
		type = policy.value.type
	return _("SOAC7514I") %{ '1' : policy.requesterIdentifier,
	       '2' : policy.serviceIdentifier, '3' : policy.operation,
	        '4' : policy.name, '5' : value, '6' : type }

def createPolicies(policy_admin, policies):
	"""Creates a list of policy definitions in the service policy manager.

	This list of policies contains a list of policy data structures as
	generated by WSDL2Java from the Policy Administration WSDL interface.
	'policy_admin' is a reference to the Policy Administration interface
	proxy."""
	log(_("SOAC7509I"))
	scoped_policies = ScopedPolicies(scopedPolicy=policies)
	status = None
	try:
		status = SPMJython.invoke(policy_admin.getRef(), "createPolicies",
		    CrtPolReq(scopedPolicies=scoped_policies))
	except MBeanException, me:
		svc_ex = me.getTargetException()
		msgid = getMessageFromTargetEx(svc_ex)
		log(_("SOAC7512E") %{ '1' : msgid })
	if status and status.faultStatus and \
	   status.faultStatus.faultStatus is not None and \
	   not None in status.faultStatus.faultStatus:
		status = status.faultStatus.faultStatus
		status = [ status[i].getFault() for i in range(0, len(status)) ]
		log(_("SOAC7510I"))
		for policy, ex in zip(policies, status):
			result = _("SOAC7513I")
			if ex:
				result = _("SOAC7515E") %{ '1' : ex.getMessageID(), '2' : ex.getText(), '3' : ex.getVariables() }
			log(_("SOAC7511I")
			    %{ '1' : policyToString(policy), '2' : result })
	else:
		log(_("SOAC7512E"))

def registerPolicyType(type_admin, type_name, editable, edit_mask,
                       encrypted, desc):
	"""Registers a policy type with the service policy manager.

	This can be used for registering custom policy types, used for
	validating policy values. The 'edit_mask' corresponds to the
	regular expression used for validating the form factor of the
	policy value contents. 'type_admin' should contain a reference
	to a type administration MBean instance."""
	log(_("SOAC7517I") %{ '1' : type_name, '2' : editable,
	    '3' : edit_mask, '4' : encrypted, '5' : desc })
	new_type = PolicyType(typeName=type_name, editable=editable,
	                      editMask=edit_mask, encrypted=encrypted,
	                      description=desc)
	try:
		SPMJython.invoke(type_admin.getRef(), "createPolicyType", CrtTypeReq(type=new_type))
		log(_("SOAC7519I"))
	except MBeanException, me:
		svc_ex = me.getTargetException()
		msgid = getMessageFromTargetEx(svc_ex)
		log(_("SOAC7518E") %{ '1' : msgid })

# Service type constants
SERVICE = ServiceType.Service
SERVICE_GROUP = ServiceType.ServiceGroup

def createService(svc_admin, service, parent_group, enabled,
                  description, type):
	"""Creates a new service definition with the service policy
	manager.
	
	This is used for creating new service definitions within the
	service hierarchy. The 'service' corresponds to the service
	identifier name and the 'parent_group' corresponds to the
	parent where the service will be linked into the hierarchy.
	The 'type' corresponds to the type of node, and must be
	either a SERVICE or SERVICE_GROUP constant. 'svc_admin'
	should contain a reference to the service administration
	MBean instance."""
	log(_("SOAC7524I") %{ '1' : service, '2' : parent_group,
	    '3' : boolToString(enabled), '4' : description,
	    '5' : type.toString() })
	new_svc = ServiceDefinition(
	    service=service, parentGroup=parent_group, enabled=enabled,
	    description=description, definitionType=type)
	try:
		SPMJython.invoke(svc_admin.getRef(), "createService", CrtSvcReq(definition=new_svc))
		log(_("SOAC7525I"))
	except MBeanException, me:
		svc_ex = me.getTargetException()
		msgid = getMessageFromTargetEx(svc_ex)
		log(_("SOAC7526E") %{ '1' : msgid })

def createOperation(svc_admin, service, operation, enabled,
                    description):
	"""Creates a new operation definition with the service policy
	manager.

	This is used to define a new operation definition for the
	specified 'service'. 'svc_admin' should contain a reference
	to the service administration MBean instance."""
	log(_("SOAC7527I") %{ '1' : service, '2': operation,
	    '3' : boolToString(enabled), '4': description })
	new_op = OperationDefinition(service=service, operation=operation,
	    enabled=enabled, description=description)
	try:
		SPMJython.invoke(svc_admin.getRef(), "createOperation", CrtOpReq(definition=new_op))
		log(_("SOAC7528I"))
	except MBeanException, me:
		svc_ex = me.getTargetException()
		msgid = getMessageFromTargetEx(svc_ex)
		log(_("SOAC7529E") %{ '1' : msgid })
                
def initialize(dbType, dbServer, dbPort, dbAlias, dbName, userName, password):
   ji = SPMJython(dbType=dbType, 
              dbServer=dbServer,
              dbPort=dbPort,
              dbAlias=dbAlias,
              dbName=dbName,
              userName=userName,
              password=password
              )
              
   ji.initialize()
   return
   
def isInitialized():
   if SPMJython.isInitialized() == 0:
      return "false"
   return "true"

def backupTable(tableName, printWriter):
   return SPMJython.backupTable(tableName, printWriter)
   
def executeDDL(fileName):
   return SPMJython.executeDDL(fileName)
   
def setConfigProperty(key, value):
   return SPMJython.setConfigProperty(key, value)

def getConfigProperty(key):
   return SPMJython.getConfigProperty(key)

def removeConfigProperty(key):
   return SPMJython.removeConfigProperty(key)
   
def isValidSchema(name):
   return SPMJython.isValidSchema(name)

